require 'securerandom'

module RailsOtp
  def generate_otp
    self.verified_status = false
    #TODO: add expiry time to config
    self.otp_expiry_time = RailsOtp::configuration.otp_expiry_time #code is valid only for 5 minutes
    self.otp_code = SecureRandom.hex(4)
  end

  def generate_otp!
    generate_otp
    save
  end

  def verify_otp(otp)
    otp_code_valid?(otp) && code_not_expired
  end

  def regenerate_otp
    generate_otp
  end

  def regenerate_otp!
    regenerate_otp
    save
  end

  private

    #check whether both generated and entered otp match or not
    def matches?(otp)
      otp == self.otp_code 
    end

    #function to check whether otp is expired or not
    def expired?
      self.otp_expiry_time < Time.now
    end

    # custom messages for invalid code and code expiry time.
    def otp_code_valid?(otp)
      if matches?(otp)
        true
      else
        errors.add(:base, "you have entered a invalid code")
        false
      end
    end

    def code_not_expired
      if !expired?
        true
      else
        errors.add(:base, "this otp code has been expired")
        false
      end
    end
end